@@ -8,17 +8,10 @@ Produce a list of extracted information, which includes:
 
 import csv
 import re
-from collections import namedtuple, Counter
-from typing import (
-    Any,
-    Mapping,
-    Optional,
-    NamedTuple,
-    Sequence,
-    MutableMapping,
-)
+from collections import Counter
+from typing import Any, Mapping, MutableMapping, NamedTuple, Optional, Sequence
 
-from minknow_api.protocol_pb2 import BarcodeUserData, ProtocolRunUserInfo
+from minknow_api.protocol_pb2 import BarcodeUserData
 
 
 # Error raised when the sample sheet cannot be parsed
@@ -43,40 +36,24 @@ ParsedData = Mapping[str, Any]
 # The value is the information read from the sample sheet
 ParsedDataDict = MutableMapping[Optional[str], ParsedData]
 
+KNOWN_FIELDNAMES = {
+    "flow_cell_id",
+    "position_id",
+    "sample_id",
+    "experiment_id",
+    "alias",
+    "type",
+    "barcode",
+    "internal_barcode",
+    "external_barcode",
+}
+
 
 # Check that only expected fieldnames are present in sample sheet
-def check_fieldnames(
-    fieldnames: Sequence[str], exception_on_unknown_field: bool
-) -> None:
+def check_fieldnames(fieldnames: Sequence[str]) -> None:
     if not fieldnames:
         raise SampleSheetParseError("No columns in sample sheet")
 
-    valid_fieldnames = {
-        "flow_cell_id",
-        "position_id",
-        "sample_id",
-        "experiment_id",
-        "alias",
-        "type",
-        "barcode",
-        "internal_barcode",
-        "external_barcode",
-        "rapid_barcode",
-        "fip_barcode",
-    }
-
-    unrecognised_fieldnames = [
-        name for name in fieldnames if name not in valid_fieldnames
-    ]
-    if unrecognised_fieldnames:
-        message = "Unrecognised columns in sample sheet: " + ", ".join(
-            "'{}'".format(name) for name in unrecognised_fieldnames
-        )
-        if exception_on_unknown_field:
-            raise SampleSheetParseError(message)
-        else:
-            print(message)
-
     # Check that all fieldnames are unique
     if len(set(fieldnames)) < len(fieldnames):
         counts = Counter(fieldnames)
@@ -101,7 +78,6 @@ def check_fieldnames(
     # Should have at most one of the following sets:
     # { 'barcode' },
     # { 'internal_barcode', 'external_barcode' },
-    # { 'rapid_barcode', 'fip_barcode' }
     #
     # If any of the above sets of barcode columns names are present:
     #   - An `alias` column must be present
@@ -117,13 +93,10 @@ def check_fieldnames(
         "barcode",
         "internal_barcode",
         "external_barcode",
-        "rapid_barcode",
-        "fip_barcode",
     }
     barcoding_column_name_sets = [
         {"barcode"},
         {"internal_barcode", "external_barcode"},
-        {"rapid_barcode", "fip_barcode"},
     ]
     for barcoding_column_name_set in barcoding_column_name_sets:
         if any(name in fieldnames for name in barcoding_column_name_set):
@@ -179,7 +152,10 @@ def check_fieldnames(
 def get_key(record: Record) -> str:
     # We checked above that we had exactly one of `position_id` and `flow_cell_id` in the record
     # So one must be in the record, and the other must not be
-    assert ("position_id" in record) != ("flow_cell_id" in record)
+    if ("position_id" in record) == ("flow_cell_id" in record):
+        raise ValueError(
+            "exactly one of 'position_id' and 'flow_cell_id' must be specified in the sample sheet"
+        )
 
     return record.get("position_id") or record.get("flow_cell_id")
 
@@ -201,15 +177,21 @@ def to_sample_type(type_str: Optional[str], line_num: int) -> Optional[int]:
         )
 
 
+# "passenger info" are fields of extra context per barcode. We treat any unknown fields
+# in barcode related rows as passenger info
+def extract_passenger_info(record: Record) -> Mapping[str, str]:
+    return {k: v for k, v in record.items() if k not in KNOWN_FIELDNAMES}
+
+
 # The per-barcode information
 BarcodeInfo = NamedTuple(
     "BarcodeInfo",
     [
         ("barcode_name", str),
         ("barcode_name_internal", str),
-        ("lamp_barcode_id", Optional[str]),
         ("alias", str),
         ("type", Optional[int]),
+        ("passenger_info", Mapping[str, str]),
     ],
 )
 
@@ -258,7 +240,7 @@ def parse_record(parsed_data: ParsedDataDict, record: Record, line_num: int):
         data["barcode_info"] = {}
 
     if "barcode" in record:
-        if not re.match("barcode(\d{2})", record["barcode"]):
+        if not re.match(r"barcode(\d{2})", record["barcode"]):
             raise SampleSheetParseError(
                 "Line {}: Bad 'barcode' name '{}'; ".format(line_num, record["barcode"])
                 + "expected a name like 'barcode01'"
@@ -266,21 +248,20 @@ def parse_record(parsed_data: ParsedDataDict, record: Record, line_num: int):
 
         barcode_name = record["barcode"]
         barcode_name_internal = None
-        lamp_barcode_id = None
         barcode_key = barcode_name
 
     elif ("internal_barcode" in record) and ("external_barcode" in record):
         external_barcode = record["external_barcode"]
         internal_barcode = record["internal_barcode"]
 
-        if not re.match("internal(\d{2})", internal_barcode):
+        if not re.match(r"internal(\d{2})", internal_barcode):
             raise SampleSheetParseError(
                 "Line {}: Bad 'internal_barcode' name '{}'; ".format(
                     line_num, internal_barcode
                 )
                 + "expected a name like 'internal01'"
             )
-        if not re.match("external(\d{2})", external_barcode):
+        if not re.match(r"external(\d{2})", external_barcode):
             raise SampleSheetParseError(
                 "Line {}: Bad 'external_barcode' name '{}'; ".format(
                     line_num, external_barcode
@@ -290,31 +271,7 @@ def parse_record(parsed_data: ParsedDataDict, record: Record, line_num: int):
 
         barcode_name = external_barcode
         barcode_name_internal = internal_barcode
-        lamp_barcode_id = None
         barcode_key = (external_barcode, internal_barcode)
-
-    elif ("rapid_barcode" in record) and ("fip_barcode" in record):
-        rapid_barcode = record["rapid_barcode"]
-        fip_barcode = record["fip_barcode"]
-
-        if not re.match("barcode(\d{2})", rapid_barcode):
-            raise SampleSheetParseError(
-                "Line {}: Bad 'rapid_barcode' name '{}'; ".format(
-                    line_num, rapid_barcode
-                )
-                + "expected a name like 'barcode01'"
-            )
-
-        if not re.match("FIP(\d{2})", fip_barcode):
-            raise SampleSheetParseError(
-                "Line {}: Bad 'fip_barcode' name '{}'; ".format(line_num, fip_barcode)
-                + "expected a name like 'FIP01'"
-            )
-
-        barcode_name = rapid_barcode
-        barcode_name_internal = None
-        lamp_barcode_id = fip_barcode
-        barcode_key = (barcode_name, lamp_barcode_id)
     else:
         # No barcode data
         # Nothing more to do for this line
@@ -333,13 +290,15 @@ def parse_record(parsed_data: ParsedDataDict, record: Record, line_num: int):
                 "Line {}: Duplicate alias: '{}'".format(line_num, record["alias"])
             )
 
+    passenger_info = extract_passenger_info(record)
+
     data["barcode_info"][barcode_key] = BarcodeInfo(
         barcode_name=barcode_name,
         barcode_name_internal=barcode_name_internal,
-        lamp_barcode_id=lamp_barcode_id,
         # We know the alias exists because we checked in `check_fieldnames`
         alias=record["alias"],
         type=to_sample_type(record.get("type"), line_num),
+        passenger_info=passenger_info,
     )
 
 
@@ -350,12 +309,12 @@ def make_barcode_user_data(barcode_info: BarcodeInfo) -> BarcodeUserData:
         barcode_user_data.barcode_name = barcode_info.barcode_name
     if barcode_info.barcode_name_internal:
         barcode_user_data.barcode_name_internal = barcode_info.barcode_name_internal
-    if barcode_info.lamp_barcode_id:
-        barcode_user_data.lamp_barcode_id = barcode_info.lamp_barcode_id
     if barcode_info.alias:
         barcode_user_data.alias = barcode_info.alias
     if barcode_info.type:
         barcode_user_data.type = barcode_info.type
+    if barcode_info.passenger_info:
+        barcode_user_data.passenger_info.update(barcode_info.passenger_info)
     return barcode_user_data
 
 
@@ -394,17 +353,14 @@ def convert_parsed_data(parsed_data_dict: ParsedDataDict):
 
 
 # filename csv file name to load
-def load_sample_sheet_csv(filename, exception_on_unknown_field=True):
+def load_sample_sheet_csv(filename):
     # Dictionary containing information per hardware position
     parsed_data_dict = {}
 
     with open(filename, newline="") as csvfile:
         reader = csv.DictReader(csvfile)
 
-        check_fieldnames(
-            fieldnames=reader.fieldnames,
-            exception_on_unknown_field=exception_on_unknown_field,
-        )
+        check_fieldnames(fieldnames=reader.fieldnames)
 
         for record in reader:
             parse_record(parsed_data_dict, record, reader.line_num)
