@@ -1,52 +1,26 @@
 """
-A minimal server implementation that is compatible with
-the minkow_api.Connection class. To use this server for
-tests you should provide Servicer classes using keyword
-arguments when initialising the server. Services should
-implement all the methods that your tests require.
+DEPRECATED.
 
-Eg:
->>> import minknow_api
->>> # Create a custom service
->>> class InstanceService(minknow_api.instance_pb2_grpc.InstanceServiceServicer):
-...     def get_version_info(self, _request, _context):
-...         pass
->>> # Add the custom service to the mock server
->>> server = minknow_api.testutils.MockMinKNOWServer(
-...     instance_service=InstanceService
-... )
->>> # Start mock server for 30 seconds
->>> with server:
-...     server.wait_for_termination(timeout=30)
+This was originally intended to provide users of minknow_api with a convenient test
+harness for their code. However, it is quite strongly tied to minknow_api's own test
+directory structure, and also proved to be too inflexible to use with even the tests for
+minknow_api's own example code.
 
-There is one required method for the connection class
-InstanceService.get_version_info() which supplies the
-current version information. This is already included
-in the mock server, however if you require methods in
-the instance service then the mock InstanceService is
-importable from this module:
-
->>> from minknow_api.testutils import MockMinKNOWServer, InstanceService
->>> class MyInstanceService(InstanceService):
-...     def get_disk_space_info(self, request, context):
-...         pass
->>> # Add the custom service to the mock server
->>> server = MockMinKNOWServer(instance_service=MyInstanceService)
-
-For examples of other service implementations check the
-test cases for this mock server.
+Consider taking inspiration from python/test/mock_server.py in the minknow_api source
+repository instead.
 """
-import logging
-from collections import namedtuple
-from concurrent import futures
-from importlib import import_module
+
 import inspect
-from packaging.version import parse
+import logging
+import warnings
+from concurrent import futures
 from pathlib import Path
 
 import grpc
+from packaging.version import parse
+
 import minknow_api
-from minknow_api import _services, _optional_services
+from minknow_api import _optional_services, _services, _import_submodule, SubmoduleType
 
 LOGGER = logging.getLogger(__name__)
 VERSION = parse(minknow_api.__version__)
@@ -57,6 +31,11 @@ DEFAULT_SERVER_PORT = 0
 
 found_test_certs_dir = None
 
+warnings.warn(
+    "minknow_api.testutils is deprecated and will be removed in a future release",
+    DeprecationWarning,
+)
+
 
 def find_test_certs_dir(extra_stack_frames_up=0):
     global found_test_certs_dir
@@ -73,7 +52,7 @@ def find_test_certs_dir(extra_stack_frames_up=0):
             new_cert_root = cert_root.parent
             logging.info("%s %s", new_cert_root, cert_root)
             if new_cert_root == cert_root:
-                raise Exception("Failed to find test certficates directory")
+                raise Exception("Failed to find test certificates directory")
             cert_root = new_cert_root
 
         found_test_certs_dir = certs_dir
@@ -176,11 +155,11 @@ class MockMinKNOWServer:
             # get service name excluding '_service'
             if service.endswith("_service"):
                 service = service[: -len("_service")]
-            if service in _services:
-                self.logger.info("Using user defined service for {!r}".format(service))
+            if service in _services.keys():
+                self.logger.info(f"Using user defined service for {repr(service)}")
             else:
                 self.logger.warning(
-                    "Skipped user defined service {!r}, not recognised".format(service)
+                    f"Skipped user defined service {repr(service)}, not recognised"
                 )
 
         # Store kwargs, add 'instance_service' here as it is the
@@ -192,17 +171,17 @@ class MockMinKNOWServer:
         # Iterate the services defined in minknow_api._services
         #   adding the base Servicer unless another is specified
         #   via kwargs
-        for name, svc_list in _services.items():
-            svc_name = "{}_service".format(name)
+        for name, svc in _services.items():
+            svc_name = f"{name}_service"
             try:
-                svc_module = import_module("minknow_api.{}_pb2_grpc".format(name))
+                svc_module = _import_submodule(name, svc, SubmoduleType.PB2_GRPC)
             except ImportError:
                 if name not in _optional_services:
                     raise
             else:
                 # There should be only one entry for each service
-                for svc in svc_list:
-                    svc_servicer = "{}Servicer".format(svc)
+                for svc_class_name in svc.services:
+                    svc_servicer = f"{svc_class_name}Servicer"
 
                     # Get the user overridden module from kwargs
                     #   or fallback onto the baseclass from grpc
@@ -216,19 +195,19 @@ class MockMinKNOWServer:
                     setattr(self, svc_name, module())
 
                     # Add servicer to server
-                    func = "add_{}_to_server".format(svc_servicer)
+                    func = f"add_{svc_servicer}_to_server"
                     add_servicer_to_server_func = getattr(svc_module, func)
                     add_servicer_to_server_func(getattr(self, svc_name), self.server)
 
             grpc_creds = None
             if self.certs_path:
                 grpc_creds = make_secure_grpc_credentials(self.certs_path)
-            bound = self.server.add_secure_port("[::]:{}".format(self.port), grpc_creds)
+            bound = self.server.add_secure_port(f"[::]:{self.port}", grpc_creds)
         if bound == 0:
-            raise ConnectionError("Could not connect using port {}".format(self.port))
+            raise ConnectionError(f"Could not connect using port {self.port}")
         self.port = bound
 
-        self.logger.info("Using port {}".format(self.port))
+        self.logger.info(f"Using port {self.port}")
 
     def make_channel_credentials(self):
         certs_path = self.__dict__["certs_path"]
