@@ -1,30 +1,85 @@
 """Tools for interacting with protocols in minknow"""
 
 import collections
+import dataclasses
+from pathlib import Path
 import logging
-import typing
+from typing import Optional, List, Sequence, Tuple
 
 import grpc
 
-from minknow_api import protocol_pb2
-from minknow_api.protocol_pb2 import BarcodeUserData
-
 from .. import Connection
+from minknow_api import protocol_pb2, run_until_pb2, acquisition_pb2
+from minknow_api.manager_pb2 import FindBasecallConfigurationsResponse
+from minknow_api.protocol_pb2 import BarcodeUserData
+from minknow_api.analysis_workflows_pb2 import AnalysisWorkflowRequest
+from minknow_api.tools.any_helpers import make_float_any, make_uint64_any
 
 LOGGER = logging.getLogger(__name__)
 
 
+def find_default_simplex_model(
+    device_connection: Connection,
+    kit: str,
+    sample_rate: int,
+    available_basecall_configs: List[
+        FindBasecallConfigurationsResponse.BasecallConfiguration
+    ],
+) -> Tuple[
+    FindBasecallConfigurationsResponse.BasecallConfiguration,
+    FindBasecallConfigurationsResponse.SimplexModel,
+]:
+    default_variant = "HAC".casefold()
+
+    flow_cell_info = device_connection.device.get_flow_cell_info()
+    flow_cell_product_code = (
+        flow_cell_info.user_specified_product_code or flow_cell_info.product_code
+    )
+
+    for config in available_basecall_configs:
+        if (
+            kit in config.kits
+            and flow_cell_product_code in config.flowcells
+            and sample_rate == config.sampling_rate
+        ):
+            for simplex in config.simplex_models:
+                if simplex.variant.casefold() == default_variant:
+                    return config, simplex
+
+    raise RuntimeError(
+        f"No simplex model available for kit: {kit}, sample_rate: {sample_rate}. Searched {len(available_basecall_configs)} configs"
+    )
+
+
+def find_simplex_model(
+    available_basecall_configs: List[
+        FindBasecallConfigurationsResponse.BasecallConfiguration
+    ],
+    simplex_model_name: str,
+) -> Tuple[FindBasecallConfigurationsResponse.SimplexModel]:
+
+    searched_configs = set()
+    for config in available_basecall_configs:
+        for simplex in config.simplex_models:
+            searched_configs.add(simplex.name)
+            if simplex.name == simplex_model_name:
+                return simplex
+
+    raise RuntimeError(
+        f"No simplex model called '{simplex_model_name}' found. Searched: {', '.join(searched_configs)} configs"
+    )
+
+
 def find_protocol(
     device_connection: Connection,
     product_code: str,
     kit: str,
-    basecalling: bool = False,
-    basecall_config: typing.Optional[str] = None,
+    config_name: Optional[str],
     barcoding: bool = False,
-    barcoding_kits: typing.Optional[typing.List[str]] = None,
+    barcoding_kits: Optional[List[str]] = None,
     force_reload: bool = False,
     experiment_type: str = "sequencing",
-) -> typing.Optional[str]:
+) -> Optional[str]:
     """Find a protocol identifier.
 
     This will fetch a list of protocols from the device-instance, then search through the protocols
@@ -35,9 +90,7 @@ def find_protocol(
         device_connection (:obj:`Connection`):  As returned by minknow.manager.FlowCellPosition().connect().
         product_code (:obj:`str`):              The flow-cell type, as in flow_cell_info.product_code.
         kit (:obj:`str`):                       The kit to be sequenced. eg: "SQK-LSK108".
-        basecalling (bool):                     True if base-calling is required
-        basecall_config (:obj:`str):            The base-calling model that the protocol should support. If absent,
-                                                the protocol default will be used (if specified)
+        config_name (:obj:`str`):               Optional name of a config to select.
         barcoding (bool):                       True if barcoding is required.
         barcoding_kits (:obj:`list`):           Barcoding kits that the protocol should support. If specified,
                                                 barcoding is assumed to be True.
@@ -68,6 +121,15 @@ def find_protocol(
         # the tags provide a way to filter the experiments
         tags = protocol.tags
 
+        # ...with the correct name...
+        if config_name is not None and protocol.name != config_name:
+            LOGGER.debug(
+                "Protocol is not named correctly %s, not %s",
+                protocol.name,
+                config_name,
+            )
+            continue
+
         # want a sequencing experiment...
         if experiment_type and tags["experiment type"].string_value != experiment_type:
             LOGGER.debug(
@@ -100,8 +162,6 @@ def find_protocol(
                 LOGGER.debug("Protocol does not support barcoding")
                 continue
             else:
-                if basecalling:
-                    LOGGER.warning("Not using barcoding for a barcoding kit")
                 LOGGER.debug("Protocol requires barcoding")
 
         supported_kits = tags["barcoding kits"].array_value
@@ -121,23 +181,6 @@ def find_protocol(
             )
             continue
 
-        # check base-calling is supported if required and with the requested model if
-        # specified or default if not
-        basecalling_required = basecalling or basecall_config is not None
-        if basecalling_required:
-            if not "default basecall model" in tags:
-                continue
-            default_basecall_model = tags.get("default basecall model").string_value
-            if not basecall_config:
-                basecall_config = default_basecall_model
-            if basecall_config not in tags["available basecall models"].array_value:
-                LOGGER.debug(
-                    'basecalling model "%s" is not in the list of supported basecalling models %s',
-                    basecall_config,
-                    tags["available basecall models"].array_value,
-                )
-                continue
-
         # we have a match, (ignore the rest)
         return protocol
     return None
@@ -149,17 +192,55 @@ BarcodingArgs = collections.namedtuple(
         "kits",
         "trim_barcodes",
         "barcodes_both_ends",
-        "detect_mid_strand_barcodes",
-        "min_score",
-        "min_score_rear",
-        "min_score_mid",
     ],
 )
 AlignmentArgs = collections.namedtuple("AlignmentArgs", ["reference_files", "bed_file"])
 BasecallingArgs = collections.namedtuple(
-    "BasecallingArgs", ["config", "barcoding", "alignment"]
+    "BasecallingArgs",
+    [
+        "simplex_model",
+        "modified_models",
+        "stereo_model",
+        "barcoding",
+        "alignment",
+        "min_qscore",
+    ],
 )
-OutputArgs = collections.namedtuple("OutputArgs", ["reads_per_file"])
+OutputArgs = collections.namedtuple("OutputArgs", ["reads_per_file", "batch_duration"])
+
+
+@dataclasses.dataclass
+class CriteriaValues:
+    """
+    Python representation of a `minknow_api.run_until.CriteriaValues` message
+
+    See the `Standard Run-Until Criteria` in `run_until.proto` for further descriptions of the fields
+    """
+
+    # Runtime, in seconds
+    runtime: Optional[int] = None
+    # Estimated bases, in bases
+    estimated_bases: Optional[int] = None
+    # Passed basecalled bases, in seconds
+    passed_basecalled_bases: Optional[int] = None
+    # Available pores
+    available_pores: Optional[float] = None
+
+    def as_protobuf(self):
+        criteria_dict = {}
+        if self.runtime is not None:
+            criteria_dict["runtime"] = make_uint64_any(self.runtime)
+        if self.estimated_bases is not None:
+            criteria_dict["estimated_bases"] = make_uint64_any(self.estimated_bases)
+        if self.passed_basecalled_bases is not None:
+            criteria_dict["passed_basecalled_bases"] = make_uint64_any(
+                self.passed_basecalled_bases
+            )
+        if self.available_pores is not None:
+            criteria_dict["available_pores"] = make_float_any(self.available_pores)
+
+        return run_until_pb2.CriteriaValues(criteria=criteria_dict)
+
 
 ReadUntilArgs = collections.namedtuple(
     "ReadUntilArgs",
@@ -179,7 +260,6 @@ ReadUntilArgs = collections.namedtuple(
 
 
 def make_protocol_arguments(
-    experiment_duration: float = 72,
     basecalling: BasecallingArgs = None,
     read_until: ReadUntilArgs = None,
     fastq_arguments: OutputArgs = None,
@@ -188,15 +268,15 @@ def make_protocol_arguments(
     bam_arguments: OutputArgs = None,
     disable_active_channel_selection: bool = False,
     mux_scan_period: float = 1.5,
-    args: typing.Optional[typing.List[str]] = None,
+    simulation_path: Optional[Path] = None,
+    args: Optional[List[str]] = None,
     is_flongle: bool = False,
-) -> typing.List[str]:
+) -> List[str]:
     """Build arguments to be used when starting a protocol.
 
     This will assemble the arguments passed to this script into arguments to pass to the protocol.
 
     Args:
-        experiment_duration(float):             Length of the experiment in hours.
         basecalling(:obj:`BasecallingArgs`):    Arguments to control basecalling.
         read_until(:obj:`ReadUntilArgs):        Arguments to control read until.
         fastq_arguments(:obj:`OutputArgs`):     Control fastq file generation.
@@ -205,6 +285,7 @@ def make_protocol_arguments(
         bam_arguments(:obj:`OutputArgs`):       Control bam file generation.
         disable_active_channel_selection(bool): Disable active channel selection
         mux_scan_period(float):                 Period of time between mux scans in hours.
+        simulation_path(:obj:`Path`):           An optional fast5 bulk path to use for simulated playback.
         args(:obj:`list`):                      Extra arguments to pass to protocol.
         is_flongle(bool):                       Specify if the flow cell to be sequenced on is a flongle.
 
@@ -223,8 +304,21 @@ def make_protocol_arguments(
     if basecalling:
         protocol_args.append("--base_calling=on")
 
-        if basecalling.config:
-            protocol_args.append("--guppy_filename=" + basecalling.config)
+        models_args = []
+
+        models_args.append(f"simplex_model='{basecalling.simplex_model}'")
+        if basecalling.modified_models:
+            modified_models = ",".join(f"'{m}'" for m in basecalling.modified_models)
+            models_args.append(f"modified_models=[{modified_models}]")
+        if basecalling.stereo_model:
+            models_args.append(f"stereo_model='{basecalling.stereo_model}'")
+
+        protocol_args.append("--basecaller_models")
+        protocol_args.extend(models_args)
+
+        protocol_args.extend(
+            ["--read_filtering", f"min_qscore={basecalling.min_qscore}"]
+        )
 
         if basecalling.barcoding:
             barcoding_args = []
@@ -248,31 +342,6 @@ def make_protocol_arguments(
                     + on_off(basecalling.barcoding.barcodes_both_ends)
                 )
 
-            if basecalling.barcoding.detect_mid_strand_barcodes:
-                # detect_mid_strand_barcodes=on/off
-                barcoding_args.append(
-                    "detect_mid_strand_barcodes="
-                    + on_off(basecalling.barcoding.detect_mid_strand_barcodes)
-                )
-
-            if basecalling.barcoding.min_score:
-                # min_score=66
-                barcoding_args.append(
-                    "min_score={}".format(basecalling.barcoding.min_score)
-                )
-
-            if basecalling.barcoding.min_score_rear:
-                # min_score_rear=66
-                barcoding_args.append(
-                    "min_score_rear={}".format(basecalling.barcoding.min_score_rear)
-                )
-
-            if basecalling.barcoding.min_score_mid:
-                # min_score_mid=66
-                barcoding_args.append(
-                    "min_score_mid={}".format(basecalling.barcoding.min_score_mid)
-                )
-
             protocol_args.extend(["--barcoding"] + barcoding_args)
 
         if basecalling.alignment:
@@ -308,40 +377,55 @@ def make_protocol_arguments(
         if read_until.last_channel:
             read_until_args.append("last_channel={}".format(read_until.last_channel))
 
-        # --read_until filter_type='enrich' reference_files=['/data/my-alignment-file'] bed_file='/data/bed_file.bed' first_channel=1 last_channel=512
-        print(read_until_args)
         protocol_args.extend(["--read_until"] + read_until_args)
 
-    protocol_args.append("--experiment_time={}".format(experiment_duration))
     protocol_args.append("--fast5=" + on_off(fast5_arguments))
     if fast5_arguments:
         protocol_args.extend(
             ["--fast5_data", "trace_table", "fastq", "raw", "vbz_compress"]
         )
-        protocol_args.append(
-            "--fast5_reads_per_file={}".format(fast5_arguments.reads_per_file)
-        )
+        if fast5_arguments.reads_per_file is not None:
+            protocol_args.append(
+                "--fast5_reads_per_file={}".format(fast5_arguments.reads_per_file)
+            )
+        if fast5_arguments.batch_duration is not None:
+            protocol_args.append(
+                "--fast5_batch_duration={}".format(fast5_arguments.batch_duration)
+            )
 
     protocol_args.append("--pod5=" + on_off(pod5_arguments))
     if pod5_arguments:
-        protocol_args.append(
-            "--pod5_reads_per_file={}".format(pod5_arguments.reads_per_file)
-        )
+        if pod5_arguments.reads_per_file is not None:
+            protocol_args.append(
+                "--pod5_reads_per_file={}".format(pod5_arguments.reads_per_file)
+            )
+        if pod5_arguments.batch_duration is not None:
+            protocol_args.append(
+                "--pod5_batch_duration={}".format(pod5_arguments.batch_duration)
+            )
 
     protocol_args.append("--fastq=" + on_off(fastq_arguments))
     if fastq_arguments:
         protocol_args.extend(["--fastq_data", "compress"])
-        protocol_args.append(
-            "--fastq_reads_per_file={}".format(fastq_arguments.reads_per_file)
-        )
+        if fastq_arguments.reads_per_file is not None:
+            protocol_args.append(
+                "--fastq_reads_per_file={}".format(fastq_arguments.reads_per_file)
+            )
+        if fastq_arguments.batch_duration is not None:
+            protocol_args.append(
+                "--fastq_batch_duration={}".format(fastq_arguments.batch_duration)
+            )
 
     protocol_args.append("--bam=" + on_off(bam_arguments))
     if bam_arguments:
-        if bam_arguments.reads_per_file != 4000:
-            raise Exception("Unable to change reads per file for BAM.")
-        """protocol_args.append(
-            "--bam_reads_per_file={}".format(bam_arguments.reads_per_file)
-        )"""
+        if bam_arguments.reads_per_file is not None:
+            protocol_args.append(
+                "--bam_reads_per_file={}".format(bam_arguments.reads_per_file)
+            )
+        if bam_arguments.batch_duration is not None:
+            protocol_args.append(
+                "--bam_batch_duration={}".format(bam_arguments.batch_duration)
+            )
 
     if not is_flongle:
         protocol_args.append(
@@ -350,19 +434,74 @@ def make_protocol_arguments(
         if not disable_active_channel_selection:
             protocol_args.append("--mux_scan_period={}".format(mux_scan_period))
 
-    protocol_args.extend(args)
+    if simulation_path:
+        if not simulation_path.exists():
+            raise Exception(
+                f"Non-existent path '{simulation_path}' passed for simulation."
+            )
+        protocol_args.extend(["--simulation", str(simulation_path)])
+
+    if args:
+        protocol_args.extend(args)
 
     return protocol_args
 
 
+def make_target_run_until_criteria(
+    stop_criteria: Optional[CriteriaValues] = None,
+    experiment_duration: Optional[float] = None,
+) -> acquisition_pb2.TargetRunUntilCriteria:
+    """Make an `acquisition.TargetRunUntilCriteria` message based on the supplied parameters
+
+    If `stop_criteria` is supplied, then the `stop_criteria` in the returned message are set to match the supplied
+    criteria.  Otherwise, if `experiment_duration` is supplied, then the "runtime" stop criterion in the returned
+    message is set to match the supplied `experiment_duration`.
+
+    If no arguments are supplied, the returned message will be empty.  If both arguments are supplied, a `ValueError`
+    is raised, since only one or the other argument may be supplied
+
+    Args:
+        stop_criteria(:obj:`CriteriaValues`):   Stop criteria to set.
+        experiment_duration(float):             Length of the experiment in hours.
+
+    Returns:
+        An `acquisition.TargetRunUntilCriteria` message with the specified criteria
+    """
+
+    if stop_criteria and experiment_duration:
+        raise ValueError(
+            "Can specify `stop_criteria` or `experiment_duration` but not both"
+        )
+
+    if experiment_duration:
+        # Case of having both is handled above
+        # The assert below corresponds to the comment above, and is NOT a run-time check
+        assert not stop_criteria  # nosec B101
+
+        # `experiment_duration` is in hours
+        # `runtime` is in seconds
+        stop_criteria = CriteriaValues(runtime=int(experiment_duration * 60 * 60))
+
+    if stop_criteria:
+        return acquisition_pb2.TargetRunUntilCriteria(
+            stop_criteria=stop_criteria.as_protobuf()
+        )
+    else:
+        # Empty target criteria
+        return acquisition_pb2.TargetRunUntilCriteria()
+
+
 def start_protocol(
     device_connection: Connection,
     identifier: str,
     sample_id: str,
     experiment_group: str,
-    barcode_info: typing.Optional[typing.Sequence[BarcodeUserData]],
+    barcode_info: Optional[Sequence[BarcodeUserData]],
+    stop_criteria: Optional[CriteriaValues] = None,
+    experiment_duration: Optional[float] = None,
+    analysis_workflow_request: Optional[AnalysisWorkflowRequest] = None,
     *args,
-    **kwargs
+    **kwargs,
 ) -> str:
     """Start a protocol on the passed {device_connection}.
 
@@ -373,11 +512,18 @@ def start_protocol(
         experiment_group(str):                  Experiment group of protocol to start.
         barcode_info(Sequence[:obj:`BarcodeUserData`]):
                 Barcode user data (sample type and alias)
+        analysis_workflow_request: Optional[AnalysisWorkflowRequest]:
+                Analysis workflow request message
+        stop_criteria(::obj::`TargetCriteria`): When to stop the acquisition
+        experiment_duration(float):             Length of the experiment in hours.
         *args: Additional arguments forwarded to {make_protocol_arguments}
         **kwargs: Additional arguments forwarded to {make_protocol_arguments}
 
     Returns:
         The protocol_run_id of the started protocol.
+
+    Notes:
+        Only one of `stop_criteria` and `experiment_duration` may be specified
     """
 
     flow_cell_info = device_connection.device.get_flow_cell_info()
@@ -395,8 +541,23 @@ def start_protocol(
     if barcode_info:
         user_info.barcode_user_info.extend(barcode_info)
 
+    # Run until criteria
+    target_run_until_criteria = make_target_run_until_criteria(
+        stop_criteria=stop_criteria,
+        experiment_duration=experiment_duration,
+    )
+
+    # Only pass analysis_workflow_request to start_protocol() if the user explicitly supplied it
+    additional_params = {}
+    if analysis_workflow_request is not None:
+        additional_params["analysis_workflow_request"] = analysis_workflow_request
+
     result = device_connection.protocol.start_protocol(
-        identifier=identifier, args=protocol_arguments, user_info=user_info
+        identifier=identifier,
+        args=protocol_arguments,
+        user_info=user_info,
+        target_run_until_criteria=target_run_until_criteria,
+        **additional_params,
     )
 
     return result.run_id
